package be.iminds.ilabt.jfed.restapi;

import be.iminds.ilabt.jfed.fedmon.util.Clock;
import be.iminds.ilabt.jfed.restapi.dao.TestPostgresDB;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.Admin;
import be.iminds.ilabt.jfed.restapi.client.JFedRestApiClient;
import be.iminds.ilabt.jfed.restapi.client.JFedRestApiClientConfig;
import be.iminds.ilabt.jfed.restapi.client.JFedRestApiClientImpl;
import be.iminds.ilabt.jfed.restapi.client.JFedRestApiClientPropertiesConfig;
import be.iminds.ilabt.jfed.restapi.json.Experiment;
import be.iminds.ilabt.jfed.restapi.json.ExperimentBuilder;
import be.iminds.ilabt.jfed.util.IOUtils;
import be.iminds.ilabt.jfed.util.JFedTrustStore;
import be.iminds.ilabt.jfed.util.KeyUtil;
import io.dropwizard.testing.junit.DropwizardAppRule;
import org.junit.*;
import sun.nio.ch.IOUtil;

import java.io.IOException;
import java.security.PrivateKey;
import java.security.cert.X509Certificate;
import java.util.Collections;
import java.util.List;
import java.util.Optional;
import java.util.Properties;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.*;

import static org.mockito.Mockito.mock;

/**
 * ExperimentIntegrationTest: temporary test using SSL server and credential.
 */
public class ExperimentIntegrationTest {
    @ClassRule
    public static final DropwizardAppRule<JFedRestApiConfiguration> RULE =
            new DropwizardAppRule<JFedRestApiConfiguration>(JFedRestApiApplication.class, "../jfed-rest-api-TESTING-SSL.yml");

//    protected static ExperimentTestCollection experimentTestCollection;

    protected JFedRestApiClient client;

    @BeforeClass
    public static void setUp() throws Exception {
        setUp(RULE);
    }
    public static void setUp(DropwizardAppRule<JFedRestApiConfiguration> RULE) throws Exception {
        Clock.set(new Clock.DefaultTestTimeProvider());

        assert RULE != null;
    }

    @AfterClass
    public static void tearDown() throws Exception {
        Clock.reset();
    }

    @Before
    public void beforeMethod() throws Exception {
        JFedTrustStore jFedTrustStore = new JFedTrustStore();
        String serverPem = IOUtils.fileToString("/home/wim/git/jfed-rest-api/generate-certs/ca/intermediate/certs/ca-chain.cert.pem");
//        List<X509Certificate> serverCerts = KeyUtil.pemToX509CertificateChain(serverPem);
        jFedTrustStore.addTrustedPemCertificate(serverPem);

        String userLoginPem = IOUtils.fileToString("/home/wim/git/jfed-rest-api/generate-certs/java/client.pem");
        X509Certificate userCert = KeyUtil.pemToX509Certificate(userLoginPem);
        PrivateKey userPrivKey = KeyUtil.pemToAnyPrivateKey(userLoginPem, null);

        Properties properties = new Properties();
        properties.setProperty("jfed_restapi_client_url_read_base", String.format("https://localhost:%d/", RULE.getLocalPort()));
        properties.setProperty("jfed_restapi_client_url_write_base", String.format("https://localhost:%d/", RULE.getLocalPort()));

        JFedRestApiClientConfig clientConfig =
                new JFedRestApiClientPropertiesConfig(
                        properties,
                        jFedTrustStore,
                        Collections.singletonList(userCert),
                        userPrivKey);
        client = new JFedRestApiClientImpl(clientConfig);
        assert client != null;

        TestPostgresDB testPostgresDB = new TestPostgresDB(RULE);
        testPostgresDB.refillDB();
        testPostgresDB.close();
    }

    @Test
    public void testDummyGet() throws Exception {
        Experiment actual = client.getById(Experiment.class, 5000l).get();

        System.out.println("Got result Experiment: "+actual);
    }

    public Experiment createExperimentForTest() throws IOException {
        String credentialsXmlString = IOUtils.fileToString("/home/wim/git/jfed-rest-api/credential-ftester_speaksfor_wvdemeer.xml");

        ExperimentBuilder experimentBuilder = new ExperimentBuilder();
//        experimentBuilder.setName("test1");
        experimentBuilder.setProject("fed4fire");
        experimentBuilder.setRequestRspec("<rspec xmlns=\"http://www.geni.net/resources/rspec/3\" " +
                "      type=\"request\" " +
                "      generated_by=\"jFed RSpec Editor\" generated=\"2016-05-24T11:35:29.953+02:00\" " +
                "      xmlns:emulab=\"http://www.protogeni.net/resources/rspec/ext/emulab/1\" " +
                "      xmlns:jfedBonfire=\"http://jfed.iminds.be/rspec/ext/jfed-bonfire/1\" " +
                "      xmlns:delay=\"http://www.protogeni.net/resources/rspec/ext/delay/1\" " +
                "      xmlns:jfed-command=\"http://jfed.iminds.be/rspec/ext/jfed-command/1\" " +
                "      xmlns:client=\"http://www.protogeni.net/resources/rspec/ext/client/1\" " +
                "      xmlns:jfed-ssh-keys=\"http://jfed.iminds.be/rspec/ext/jfed-ssh-keys/1\" " +
                "      xmlns:jfed=\"http://jfed.iminds.be/rspec/ext/jfed/1\" " +
                "      xmlns:sharedvlan=\"http://www.protogeni.net/resources/rspec/ext/shared-vlan/1\" " +
                "      xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" " +
                "      xsi:schemaLocation=\"http://www.geni.net/resources/rspec/3 http://www.geni.net/resources/rspec/3/request.xsd \">\n" +
                "  <node client_id=\"node0\" exclusive=\"true\" component_manager_id=\"urn:publicid:IDN+wall2.ilabt.iminds.be+authority+cm\">\n" +
                "    <sliver_type name=\"please-fail\"/>\n" +
                "  </node>\n" +
                "</rspec>");
        experimentBuilder.setSpeaksForCredential(credentialsXmlString);

        return experimentBuilder.create();
    }

    @Test
    public void testPrintExperimentRequest() throws Exception {
        Experiment createRequest = createExperimentForTest();

        System.out.println("Experiment request: " + createRequest);
    }

    @Test
    public void testCreate() throws Exception {
        Experiment createRequest = createExperimentForTest();

        Experiment created = client.create(createRequest);

        System.out.println("Got created Experiment: "+created);


        int count = 0;
        boolean done = false;
        while (!done) {
            Thread.sleep(1000l);

            Optional<Experiment> statusUpdate = client.getById(Experiment.class, created.getId());

            assertThat(statusUpdate.isPresent(), is(true));

            System.out.println("Got statusUpdate Experiment: " + statusUpdate.get());

            switch (statusUpdate.get().getStatus()) {
                case STARTING: { done = false; break; }
                case FAILED: { done = true; break; }
                case NONEXISTING: { done = true; break; }
                case READY: { done = true; break; }
                default: throw new RuntimeException("Unsupported status: "+statusUpdate.get().getStatus());
            }

            if (done) {
                assertThat(statusUpdate.get().getStatus(), is(Experiment.Status.FAILED));
            }

            count++;
            assertThat(count, is(lessThan(20)));
        }
    }
}
