package be.iminds.ilabt.jfed.restapi.debugdata;

import be.iminds.ilabt.jfed.util.RFC3339Util;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.sql.Timestamp;

/**
 * LogLineDD
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonIgnoreProperties("@context")
public class LogLineDD implements DebugData {
    private static final Logger LOG = LoggerFactory.getLogger(LogLineDD.class);

    private final String level;
    private final Timestamp date;
    private final String thread;
    private final String loggerName;
    private final String message;
    private final String stacktrace;

    @JsonCreator
    public LogLineDD(@JsonProperty("level") String level,
                     @JsonProperty("date") Timestamp date,
                     @JsonProperty("thread") String thread,
                     @JsonProperty("loggerName") String loggerName,
                     @JsonProperty("message") String message,
                     @JsonProperty("stacktrace") String stacktrace) {
        this.level = level;
        this.date = date;
        this.thread = thread;
        this.loggerName = loggerName;
        this.message = message;
        this.stacktrace = stacktrace;
    }

    @JsonProperty
    public String getLevel() {
        return level;
    }

    @JsonProperty
    @JsonDeserialize(using = RFC3339Util.TimeStampJsonDeserializer.class)
    @JsonSerialize(using = RFC3339Util.TimeStampJsonSerializer.class)
    public Timestamp getDate() {
        return date;
    }

    @JsonProperty
    public String getThread() {
        return thread;
    }

    @JsonProperty
    public String getLoggerName() {
        return loggerName;
    }

    @JsonProperty
    public String getMessage() {
        return message;
    }

    @JsonProperty
    public String getStacktrace() {
        return stacktrace;
    }


    @JsonProperty(value="@type")
    public String getClassName() {
        return getClass().getSimpleName();
    }

    @JsonProperty(value="@type")
    private void setClassName(String className) {
        if (!getClass().getSimpleName().equals(className))
            throw new IllegalArgumentException("@type must be "+getClass().getSimpleName());
    }

    @Override
    public String toString() {
        ObjectMapper mapper = new ObjectMapper();
        try {
            return mapper.writerWithDefaultPrettyPrinter().writeValueAsString(this);
        } catch (JsonProcessingException e) {
            LOG.error("Exception converting "+getClass().getSimpleName()+" to JSON", e);
            return "Exception converting "+ getClass().getSimpleName() +" to JSON: "+e.getMessage();
        }
    }
}
